@@ -117,6 +117,8 @@ static void          gcal_event_initable_iface_init              (GInitableIface
 G_DEFINE_TYPE_WITH_CODE (GcalEvent, gcal_event, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, gcal_event_initable_iface_init))
 
+G_DEFINE_QUARK (GcalEvent, gcal_event_error);
+
 enum {
   PROP_0,
   PROP_ALL_DAY,
@@ -135,6 +137,8 @@ enum {
   N_PROPS
 };
 
+static GParamSpec* properties[N_PROPS] = { NULL, };
+
 
 /*
  * GcalEvent cache
@@ -157,6 +161,8 @@ destroy_event_cache_map (void)
 {
   GList *events;
 
+  g_debug ("Number of cached events at destruction: %d", g_hash_table_size (event_cache));
+
   /* Destroy all events */
   events = g_hash_table_get_values (event_cache);
   g_list_free_full (events, g_object_unref);
@@ -221,9 +227,8 @@ get_timezone_from_ical (ECalComponentDateTime *comp)
 }
 
 static ECalComponentDateTime*
-build_component_from_datetime (GcalEvent    *self,
-                               icaltimezone *current_tz,
-                               GDateTime    *dt)
+build_component_from_datetime (GcalEvent *self,
+                               GDateTime *dt)
 {
   ECalComponentDateTime *comp_dt;
 
@@ -231,15 +236,19 @@ build_component_from_datetime (GcalEvent    *self,
     return NULL;
 
   comp_dt = g_new0 (ECalComponentDateTime, 1);
-  comp_dt->value = NULL;
-  comp_dt->tzid = NULL;
-
   comp_dt->value = datetime_to_icaltime (dt);
-  comp_dt->value->zone = self->all_day ? icaltimezone_get_utc_timezone () : current_tz;
   comp_dt->value->is_date = self->all_day;
 
-  /* All day events have UTC timezone */
-  comp_dt->tzid = g_strdup (self->all_day ? "UTC" : icaltimezone_get_tzid (current_tz));
+  if (self->all_day)
+    {
+      comp_dt->value->zone = icaltimezone_get_utc_timezone ();
+      comp_dt->tzid = g_strdup ("UTC");
+    }
+  else
+    {
+      comp_dt->value->zone = e_cal_util_get_system_timezone ();
+      comp_dt->tzid = g_strdup (icaltimezone_get_tzid ((icaltimezone *) comp_dt->value->zone));
+    }
 
   return comp_dt;
 }
@@ -288,7 +297,7 @@ gcal_event_update_uid_internal (GcalEvent *self)
     }
 
   e_cal_component_free_id (id);
-  g_object_notify (G_OBJECT (self), "uid");
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_UID]);
 }
 
 static void
@@ -412,11 +421,11 @@ gcal_event_set_component_internal (GcalEvent     *self,
       /* Load and setup the alarms */
       load_alarms (self);
 
-      g_object_notify (G_OBJECT (self), "has-recurrence");
-      g_object_notify (G_OBJECT (self), "recurrence");
-      g_object_notify (G_OBJECT (self), "component");
-      g_object_notify (G_OBJECT (self), "location");
-      g_object_notify (G_OBJECT (self), "summary");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HAS_RECURRENCE]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_RECURRENCE]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COMPONENT]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_LOCATION]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SUMMARY]);
 
       g_clear_pointer (&zone_start, g_time_zone_unref);
       g_clear_pointer (&zone_end, g_time_zone_unref);
@@ -620,170 +629,146 @@ gcal_event_class_init (GcalEventClass *klass)
    *
    * Whether the event is all day or not.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ALL_DAY,
-                                   g_param_spec_boolean ("all-day",
-                                                         "If event is all day",
-                                                         "Whether the event is all day or not",
-                                                         FALSE,
-                                                         G_PARAM_READWRITE));
+
+  properties[PROP_ALL_DAY] = g_param_spec_boolean ("all-day",
+                                                   "If event is all day",
+                                                   "Whether the event is all day or not",
+                                                   FALSE,
+                                                   G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEvent::color:
    *
    * The color of the event.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_COLOR,
-                                   g_param_spec_boxed ("color",
-                                                       "Color of the event",
-                                                       "The color of the event",
-                                                       GDK_TYPE_RGBA,
-                                                       G_PARAM_READWRITE));
+  properties[PROP_COLOR] = g_param_spec_boxed ("color",
+                                               "Color of the event",
+                                               "The color of the event",
+                                               GDK_TYPE_RGBA,
+                                               G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEvent::component:
    *
    * The #ECalComponent of this event.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_COMPONENT,
-                                   g_param_spec_object ("component",
-                                                        "Component",
-                                                        "The ECalComponent of the event",
-                                                        E_TYPE_CAL_COMPONENT,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  properties[PROP_COMPONENT] = g_param_spec_object ("component",
+                                                     "Component",
+                                                     "The ECalComponent of the event",
+                                                     E_TYPE_CAL_COMPONENT,
+                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEvent::date-end:
    *
    * The end date of the event.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DATE_END,
-                                   g_param_spec_boxed ("date-end",
-                                                       "End date of the event",
-                                                       "The end date of the event",
-                                                       G_TYPE_DATE_TIME,
-                                                       G_PARAM_READWRITE));
+  properties[PROP_DATE_END] = g_param_spec_boxed ("date-end",
+                                                  "End date of the event",
+                                                  "The end date of the event",
+                                                  G_TYPE_DATE_TIME,
+                                                  G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEvent::date-start:
    *
    * The start date of the event.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DATE_START,
-                                   g_param_spec_boxed ("date-start",
-                                                       "Start date of the event",
-                                                       "The start date of the event",
-                                                       G_TYPE_DATE_TIME,
-                                                       G_PARAM_READWRITE));
+  properties[PROP_DATE_START] = g_param_spec_boxed ("date-start",
+                                                    "Start date of the event",
+                                                    "The start date of the event",
+                                                    G_TYPE_DATE_TIME,
+                                                    G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
    * GcalEvent::description:
    *
    * The description of the event.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DESCRIPTION,
-                                   g_param_spec_string ("description",
-                                                        "Description of the event",
-                                                        "The description of the event",
-                                                        "",
-                                                        G_PARAM_READWRITE));
-
-  /**
-   * GcalEvent::location:
-   *
-   * The location of the event.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_LOCATION,
-                                   g_param_spec_string ("location",
-                                                        "Location of the event",
-                                                        "The location of the event",
-                                                        "",
-                                                        G_PARAM_READWRITE));
-
-  /**
-   * GcalEvent::source:
-   *
-   * The #ESource this event belongs to.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_SOURCE,
-                                   g_param_spec_object ("source",
-                                                        "ESource",
-                                                        "The ESource this event belongs to",
-                                                        E_TYPE_SOURCE,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
-  /**
-   * GcalEvent::summary:
-   *
-   * The summary of the event.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_SUMMARY,
-                                   g_param_spec_string ("summary",
-                                                        "Summary of the event",
-                                                        "The summary of the event",
-                                                        "",
-                                                        G_PARAM_READWRITE));
-
-  /**
-   * GcalEvent::timezone:
-   *
-   * The timezone of the event.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_TIMEZONE,
-                                   g_param_spec_boxed ("timezone",
-                                                       "Timezone of the event",
-                                                       "The timezone of the event",
-                                                       G_TYPE_TIME_ZONE,
-                                                       G_PARAM_READWRITE));
-
-  /**
-   * GcalEvent::uid:
-   *
-   * The unique identifier of the event.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_UID,
-                                   g_param_spec_string ("uid",
-                                                        "Identifier of the event",
-                                                        "The unique identifier of the event",
-                                                        "",
-                                                        G_PARAM_READABLE));
+  properties[PROP_DESCRIPTION] = g_param_spec_string ("description",
+                                                      "Description of the event",
+                                                      "The description of the event",
+                                                      "",
+                                                      G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
   * GcalEvent::has-recurrence:
   *
   * The recurrence property of the event.
   */
-  g_object_class_install_property (object_class,
-                                   PROP_HAS_RECURRENCE,
-                                   g_param_spec_boolean ("has-recurrence",
-                                                         "If event has recurrence",
-                                                         "Whether the event has recurrence or not",
-                                                         FALSE,
-                                                         G_PARAM_READABLE));
+  properties[PROP_HAS_RECURRENCE] = g_param_spec_boolean ("has-recurrence",
+                                                          "If event has recurrence",
+                                                          "Whether the event has recurrence or not",
+                                                          FALSE,
+                                                          G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GcalEvent::location:
+   *
+   * The location of the event.
+   */
+  properties[PROP_LOCATION] = g_param_spec_string ("location",
+                                                   "Location of the event",
+                                                   "The location of the event",
+                                                   "",
+                                                   G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
   * GcalEvent::recurrence:
   *
   * The recurrence-rules property of the event.
   */
-  g_object_class_install_property (object_class,
-                                   PROP_RECURRENCE,
-                                   g_param_spec_boxed ("recurrence",
-                                                       "Recurrence property of the event",
-                                                       "The recurrence property of the event",
-                                                       GCAL_TYPE_RECURRENCE,
-                                                       G_PARAM_READWRITE));
+  properties[PROP_RECURRENCE] = g_param_spec_boxed ("recurrence",
+                                                    "Recurrence property of the event",
+                                                    "The recurrence property of the event",
+                                                    GCAL_TYPE_RECURRENCE,
+                                                    G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * GcalEvent::source:
+   *
+   * The #ESource this event belongs to.
+   */
+  properties[PROP_SOURCE] = g_param_spec_object ("source",
+                                                 "ESource",
+                                                 "The ESource this event belongs to",
+                                                 E_TYPE_SOURCE,
+                                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GcalEvent::summary:
+   *
+   * The summary of the event.
+   */
+  properties[PROP_SUMMARY] = g_param_spec_string ("summary",
+                                                        "Summary of the event",
+                                                        "The summary of the event",
+                                                        "",
+                                                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GcalEvent::timezone:
+   *
+   * The timezone of the event.
+   */
+  properties[PROP_TIMEZONE] = g_param_spec_boxed ("timezone",
+                                                  "Timezone of the event",
+                                                  "The timezone of the event",
+                                                  G_TYPE_TIME_ZONE,
+                                                  G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GcalEvent::uid:
+   *
+   * The unique identifier of the event.
+   */
+  properties[PROP_UID] = g_param_spec_string ("uid",
+                                              "Identifier of the event",
+                                              "The unique identifier of the event",
+                                              "",
+                                              G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
@@ -801,17 +786,6 @@ gcal_event_init (GcalEvent *self)
   self->is_valid = TRUE;
 }
 
-/**
- * gcal_event_error_quark:
- *
- * Error quark for event creating.
- */
-GQuark
-gcal_event_error_quark (void)
-{
-  return g_quark_from_static_string ("Invalid start date");
-}
-
 /**
  * gcal_event_new:
  * @source: (nullable): an #ESource
@@ -912,7 +886,7 @@ gcal_event_set_color (GcalEvent *self,
       g_clear_pointer (&self->color, gdk_rgba_free);
       self->color = gdk_rgba_copy (color);
 
-      g_object_notify (G_OBJECT (self), "color");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_COLOR]);
     }
 }
 
@@ -949,7 +923,7 @@ gcal_event_set_all_day (GcalEvent *self,
     {
       self->all_day = all_day;
 
-      g_object_notify (G_OBJECT (self), "all-day");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ALL_DAY]);
     }
 }
 
@@ -985,29 +959,21 @@ gcal_event_set_date_end (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_end != dt &&
-      (!self->dt_end || !dt ||
-       (self->dt_end && dt && !g_date_time_equal (self->dt_end, dt))))
+  if (self->dt_end != dt)
     {
-      ECalComponentDateTime *component_dt, current;
+      ECalComponentDateTime *component_dt;
 
       g_clear_pointer (&self->dt_end, g_date_time_unref);
       self->dt_end = g_date_time_ref (dt);
 
-      /* Retrieve the current timezone */
-      e_cal_component_get_dtstart (self->component, &current);
-
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self,
-                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
-                                                    dt);
+      component_dt = build_component_from_datetime (self, dt);
 
       e_cal_component_set_dtend (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
 
-      g_object_notify (G_OBJECT (self), "date-end");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DATE_END]);
 
-      e_cal_component_free_datetime (&current);
       e_cal_component_free_datetime (component_dt);
       g_free (component_dt);
     }
@@ -1042,29 +1008,21 @@ gcal_event_set_date_start (GcalEvent *self,
 {
   g_return_if_fail (GCAL_IS_EVENT (self));
 
-  if (self->dt_start != dt &&
-      (!self->dt_start || !dt ||
-       (self->dt_start && dt && !g_date_time_equal (self->dt_start, dt))))
+  if (self->dt_start != dt)
     {
-      ECalComponentDateTime *component_dt, current;
+      ECalComponentDateTime *component_dt;
 
       g_clear_pointer (&self->dt_start, g_date_time_unref);
       self->dt_start = g_date_time_ref (dt);
 
-      /* Retrieve the current timezone */
-      e_cal_component_get_dtstart (self->component, &current);
-
       /* Setup the ECalComponent's datetime value */
-      component_dt = build_component_from_datetime (self,
-                                                    icaltimezone_get_builtin_timezone_from_tzid (current.tzid),
-                                                    dt);
+      component_dt = build_component_from_datetime (self, dt);
 
       e_cal_component_set_dtstart (self->component, component_dt);
       e_cal_component_commit_sequence (self->component);
 
-      g_object_notify (G_OBJECT (self), "date-start");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DATE_START]);
 
-      e_cal_component_free_datetime (&current);
       e_cal_component_free_datetime (component_dt);
       g_free (component_dt);
     }
@@ -1116,7 +1074,7 @@ gcal_event_set_description (GcalEvent   *self,
       e_cal_component_set_description_list (self->component, &list);
       e_cal_component_commit_sequence (self->component);
 
-      g_object_notify (G_OBJECT (self), "description");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DESCRIPTION]);
     }
 }
 
@@ -1332,7 +1290,7 @@ gcal_event_set_location (GcalEvent   *self,
     {
       e_cal_component_set_location (self->component, location);
 
-      g_object_notify (G_OBJECT (self), "location");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_LOCATION]);
     }
 }
 
@@ -1399,7 +1357,7 @@ gcal_event_set_source (GcalEvent *self,
                                                              NULL);
         }
 
-      g_object_notify (G_OBJECT (self), "source");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SOURCE]);
 
       gcal_event_update_uid_internal (self);
     }
@@ -1452,7 +1410,7 @@ gcal_event_set_summary (GcalEvent   *self,
       e_cal_component_set_summary (self->component, &text_component);
       e_cal_component_commit_sequence (self->component);
 
-      g_object_notify (G_OBJECT (self), "summary");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SUMMARY]);
     }
 }
 
@@ -1505,13 +1463,13 @@ gcal_event_set_timezone (GcalEvent *self,
               new_dtend = g_date_time_to_timezone (self->dt_end, timezone);
               gcal_event_set_date_end (self, new_dtend);
 
-              g_clear_pointer (&new_dtstart, g_date_time_unref);
+              g_clear_pointer (&new_dtend, g_date_time_unref);
             }
 
           g_clear_pointer (&new_dtstart, g_date_time_unref);
         }
 
-      g_object_notify (G_OBJECT (self), "timezone");
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TIMEZONE]);
     }
 }
 
