@@ -1,6 +1,4 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * gcal-utils.c
+/* gcal-utils.c
  *
  * Copyright (C) 2012 - Erick Pérez Castellanos
  *
@@ -22,6 +20,7 @@
 
 #define G_LOG_DOMAIN "Utils"
 
+#include "gcal-enums.h"
 #include "gcal-utils.h"
 #include "gcal-event-widget.h"
 #include "gcal-view.h"
@@ -199,24 +198,7 @@ datetime_is_date (GDateTime *dt)
 icaltimetype*
 gcal_dup_icaltime (const icaltimetype *date)
 {
-  icaltimetype *new_date;
-
-  if (date == NULL)
-    return NULL;
-
-  new_date= g_new (icaltimetype, 1);
-  new_date->year = date->year;
-  new_date->month = date->month;
-  new_date->day = date->day;
-  new_date->hour = date->hour;
-  new_date->minute = date->minute;
-  new_date->second = date->second;
-  new_date->is_utc = date->is_utc;
-  new_date->is_date = date->is_date;
-  new_date->is_daylight = date->is_daylight;
-  new_date->zone = date->zone;
-
-  return new_date;
+  return g_memdup (date, sizeof (icaltimetype));
 }
 
 /**
@@ -264,7 +246,6 @@ gcal_get_surface_from_color (GdkRGBA  *color,
   cairo_surface_t *surface;
   cairo_t *cr;
 
-  /* TODO: review size here, maybe not hardcoded */
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, size, size);
   cr = cairo_create (surface);
 
@@ -426,16 +407,14 @@ gint
 get_first_weekday (void)
 {
   int week_start;
-#ifdef HAVE__NL_TIME_FIRST_WEEKDAY
-  union { unsigned int word; char *string; } langinfo;
-  int week_1stday = 0;
-  int first_weekday = 1;
-  guint week_origin;
-#else
-  char *gtk_week_start;
-#endif
 
 #ifdef HAVE__NL_TIME_FIRST_WEEKDAY
+
+  union { unsigned int word; char *string; } langinfo;
+  gint week_1stday = 0;
+  gint first_weekday = 1;
+  guint week_origin;
+
   langinfo.string = nl_langinfo (_NL_TIME_FIRST_WEEKDAY);
   first_weekday = langinfo.string[0];
   langinfo.string = nl_langinfo (_NL_TIME_WEEK_1STDAY);
@@ -448,7 +427,12 @@ get_first_weekday (void)
     g_warning ("Unknown value of _NL_TIME_WEEK_1STDAY.\n");
 
   week_start = (week_1stday + first_weekday - 1) % 7;
+
 #else
+
+  gchar *gtk_week_start;
+
+
   /* Use a define to hide the string from xgettext */
 # define GTK_WEEK_START "calendar:week_start:0"
   gtk_week_start = dgettext ("gtk30", GTK_WEEK_START);
@@ -464,6 +448,7 @@ get_first_weekday (void)
                  "did so wrongly.\n");
       week_start = 0;
     }
+
 #endif
 
   return week_start;
@@ -892,86 +877,6 @@ fix_popover_menu_icons (GtkPopover *popover)
   g_list_free (menu_section_box_children);
 }
 
-/**
- * uri_get_fields:
- * @uri: the URI
- * @schema: (nullable): return location for the schema of the URI
- * @host: (nullable): return location for the host of the URI
- * @path: (nullable): return location for the resource path
- *
- * Split the given URI into the fields.
- *
- * Returns: #TRUE if @uri could be parsed, #FALSE otherwise
- */
-gboolean
-uri_get_fields (const gchar  *uri,
-                gchar       **schema,
-                gchar       **host,
-                gchar       **path,
-                gboolean     *is_file)
-{
-  GRegex *regex;
-  GMatchInfo *match;
-  gboolean valid;
-
-  g_return_val_if_fail (uri != NULL, FALSE);
-
-  match = NULL;
-  valid = FALSE;
-
-  regex = g_regex_new ("([a-zA-Z0-9\\+\\.\\-]*:\\/\\/){0,1}([-a-zA-Z0-9@:%._\\+~#=]{2,256}\\.[a-z]{2,6}\\b)([-a-zA-Z0-9@:%_\\+.//=]*)",
-                       G_REGEX_CASELESS, 0, NULL);
-
-  /*
-   * Retrieved matching URI. The whole url is
-   * checked and the regex groups are:
-   * 1. schema
-   * 2. host
-   * 3. server path
-   */
-  if (g_regex_match (regex, uri, 0, &match))
-    {
-      valid = TRUE;
-
-      if (schema != NULL)
-        *schema = g_match_info_fetch (match, 1);
-
-      if (host != NULL)
-        *host = g_match_info_fetch (match, 2);
-
-      if (path != NULL)
-        *path = g_match_info_fetch (match, 3);
-
-      g_match_info_free (match);
-    }
-  else
-    {
-      if (schema != NULL)
-        *schema = NULL;
-
-      if (host != NULL)
-        *host = NULL;
-
-      if (path != NULL)
-        *path = NULL;
-    }
-
-  /* File extension */
-  if (is_file)
-    {
-      GRegex *extension_regex;
-
-      extension_regex = g_regex_new ("(\\.[a-zA-Z0-9]+)$", G_REGEX_CASELESS, 0, NULL);
-
-      *is_file = g_regex_match (extension_regex, uri, 0, NULL);
-
-      g_regex_unref (extension_regex);
-    }
-
-  g_regex_unref (regex);
-  return valid;
-}
-
 /**
  * get_source_parent_name_color:
  * @manager: a #GcalManager
